@@ -1,16 +1,17 @@
-#!/usr/bin/python -u
+#!/usr/bin/env python3
 #
 # This test exercise the redirection of error messages with a
 # functions defined in Python.
 #
 import sys
+import setup_test
 import libxml2
 
 # Memory debug specific
 libxml2.debugMemory(1)
 
 expect="""--> (3) xmlns: URI foo is not absolute
---> (4) Opening and ending tag mismatch: x line 0 and y
+--> (4) Opening and ending tag mismatch: x line 1 and y
 """
 
 err=""
@@ -23,7 +24,7 @@ s = """<x xmlns="foo"></y>"""
 parserCtxt = libxml2.createPushParser(None,"",0,"test.xml")
 parserCtxt.setErrorHandler(callback, "-->")
 if parserCtxt.getErrorHandler() != (callback,"-->"):
-    print "getErrorHandler failed"
+    print("getErrorHandler failed")
     sys.exit(1)
 parserCtxt.parseChunk(s,len(s),1)
 doc = parserCtxt.doc()
@@ -31,9 +32,9 @@ doc.freeDoc()
 parserCtxt = None
 
 if err != expect:
-    print "error"
-    print "received %s" %(err)
-    print "expected %s" %(expect)
+    print("error")
+    print("received %s" %(err))
+    print("expected %s" %(expect))
     sys.exit(1)
 
 i = 10000
@@ -50,7 +51,6 @@ while i > 0:
 # Memory debug specific
 libxml2.cleanupParser()
 if libxml2.debugMemory(1) == 0:
-    print "OK"
+    print("OK")
 else:
-    print "Memory leak %d bytes" % (libxml2.debugMemory(1))
-    libxml2.dumpMemory()
+    print("Memory leak %d bytes" % (libxml2.debugMemory(1)))
